Package: artMS
Type: Package
Title: Analytical R tools for Mass Spectrometry
Version: 1.28.0
Date: 2021-07-13
Authors@R: c(person("David", "Jimenez-Morales", role = c("aut","cre"),
                     email = "biodavidjm@gmail.com",
                     comment = c(ORCID = "0000-0003-4356-6461")),
             person("Alexandre", "Rosa Campos", role = c("aut","ctb"),
                     comment = c(ORCID = "0000-0003-3988-7764")),
             person("John", "Von Dollen", role = c("aut")),
             person("Nevan", "Krogan", role = c("aut"),
             comment = c(ORCID = "0000-0003-4902-337X")),
             person("Danielle", "Swaney", role = c("aut", "ctb"),
                     comment = c(ORCID = "0000-0001-6119-6084")))
Description: artMS provides a set of tools for the analysis of proteomics 
    label-free datasets. It takes as input the MaxQuant search result 
    output (evidence.txt file) and performs quality control, 
    relative quantification using MSstats, downstream analysis 
    and integration. artMS also provides a set of functions to re-format 
    and make it compatible with other analytical tools, including, SAINTq, 
    SAINTexpress, Phosfate, and PHOTON. Check 
    [http://artms.org](http://artms.org) for details.
License: GPL (>= 3) + file LICENSE
URL: http://artms.org
BugReports: https://github.com/biodavidjm/artMS/issues
Depends: R (>= 4.1.0)
Imports: AnnotationDbi, bit64, circlize, cluster, corrplot, data.table,
        dplyr, getopt, ggdendro, ggplot2, gplots, ggrepel, graphics,
        grDevices, grid, limma, MSstats, openxlsx, org.Hs.eg.db,
        pheatmap, plotly, plyr, RColorBrewer, scales, seqinr, stats,
        stringr, tidyr, UpSetR, utils, VennDiagram, yaml
Suggests: BiocStyle, ComplexHeatmap, factoextra, FactoMineR, gProfileR,
        knitr, PerformanceAnalytics, org.Mm.eg.db, rmarkdown, testthat
Encoding: UTF-8
LazyLoad: yes
LazyData: true
biocViews: Proteomics, DifferentialExpression, BiomedicalInformatics,
        SystemsBiology, MassSpectrometry, Annotation, QualityControl,
        GeneSetEnrichment, Clustering, Normalization, ImmunoOncology,
        MultipleComparison
NeedsCompilation: no
VignetteBuilder: knitr
RoxygenNote: 7.1.1
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/artMS
git_branch: RELEASE_3_22
git_last_commit: 9313c4c
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
Packaged: 2025-10-30 02:33:15 UTC; biocbuild
Author: David Jimenez-Morales [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4356-6461>),
  Alexandre Rosa Campos [aut, ctb] (ORCID:
    <https://orcid.org/0000-0003-3988-7764>),
  John Von Dollen [aut],
  Nevan Krogan [aut] (ORCID: <https://orcid.org/0000-0003-4902-337X>),
  Danielle Swaney [aut, ctb] (ORCID:
    <https://orcid.org/0000-0001-6119-6084>)
Maintainer: David Jimenez-Morales <biodavidjm@gmail.com>
