# Generated by roxygen2: do not edit by hand

export(artmsAnalysisQuantifications)
export(artmsAnnotateSpecie)
export(artmsAnnotationUniprot)
export(artmsAvgIntensityRT)
export(artmsChangeColumnName)
export(artmsConvertMetabolomics)
export(artmsDataPlots)
export(artmsEnrichLog2fc)
export(artmsEnrichProfiler)
export(artmsEvidenceToSAINTq)
export(artmsEvidenceToSaintExpress)
export(artmsFilterEvidenceContaminants)
export(artmsGeneratePhSiteExtended)
export(artmsIsEvidenceNewVersion)
export(artmsIsSpeciesSupported)
export(artmsLeaveOnlyUniprotEntryID)
export(artmsMapUniprot2Entrez)
export(artmsMergeEvidenceAndKeys)
export(artmsMsstatsSummary)
export(artmsPhosfateOutput)
export(artmsPhotonOutput)
export(artmsPlotHeatmapQuant)
export(artmsProtein2SiteConversion)
export(artmsQualityControlEvidenceBasic)
export(artmsQualityControlEvidenceExtended)
export(artmsQualityControlMetabolomics)
export(artmsQualityControlSummaryExtended)
export(artmsQuantification)
export(artmsResultsWide)
export(artmsSILACtoLong)
export(artmsSpectralCounts)
export(artmsVolcanoPlot)
export(artmsWriteConfigYamlFile)
import(AnnotationDbi, except = c(head))
import(MSstats)
import(RColorBrewer)
import(UpSetR)
import(VennDiagram)
import(bit64)
import(circlize)
import(data.table)
import(getopt)
import(ggdendro)
import(ggplot2)
import(ggrepel)
import(grid)
import(limma)
import(openxlsx)
import(org.Hs.eg.db)
import(pheatmap)
import(plotly, except = c(last_plot, mutate, arrange, 
rename, summarise, select, add_heatmap))
import(seqinr, except = c(zscore, count, a))
import(stringr)
import(yaml)
importFrom(cluster,pam)
importFrom(corrplot,corrplot)
importFrom(dplyr,arrange)
importFrom(dplyr,count)
importFrom(dplyr,desc)
importFrom(dplyr,everything)
importFrom(dplyr,mutate)
importFrom(dplyr,one_of)
importFrom(gplots,heatmap.2)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(graphics,barplot)
importFrom(graphics,hist)
importFrom(graphics,lines)
importFrom(graphics,pairs)
importFrom(graphics,panel.smooth)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,rect)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(plyr,ddply)
importFrom(plyr,rename)
importFrom(plyr,summarise)
importFrom(scales,percent)
importFrom(stats,IQR)
importFrom(stats,aggregate)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,dist)
importFrom(stats,fisher.test)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,order.dendrogram)
importFrom(stats,phyper)
importFrom(stats,prcomp)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,unnest)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,read.delim)
importFrom(utils,sessionInfo)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,write.table)
