% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/otherApps.R
\name{artmsPhosfateOutput}
\alias{artmsPhosfateOutput}
\title{Generate Phosfate Input file}
\usage{
artmsPhosfateOutput(inputFile, output_dir = ".", verbose = TRUE)
}
\arguments{
\item{inputFile}{(char) the \code{imputedL2fcExtended.txt} file name and location}

\item{output_dir}{(char) Name of the folder to output results
(Default: current directory. Recommended: \code{phosfate_input})}

\item{verbose}{(logical) \code{TRUE} (default) to show function messages}
}
\value{
Multiple output files (inputs of phosfate)
}
\description{
It takes as input the \code{imputedL2fcExtended.txt} results
generated by the \code{artmsAnalysisQuantifications()} function and generates
the \href{http://phosfate.com/}{Phosfate} input file (or data.frame)
Please, notice that the only species suported by Phosfate is humans.
}
\examples{
\dontrun{
artmsPhosfateOutput(inputFile)
}
}
\keyword{files}
\keyword{generate,}
\keyword{outputs,}
